'From Squeak3.9alpha of 4 July 2005 [latest update: #7006] on 22 March 2006 at 7:25:56 pm'!
"Change Set:		fastLanguageEditor-tak
Date:			13 March 2006
Author:			Takashi Yamamiya

Add instance variable 'translations' and 'untranslated' as cache for
speed up the UI operation.
"!

SystemWindow subclass: #LanguageEditor
	instanceVariableNames: 'translator selectedTranslation selectedTranslations selectedUntranslated translationsList untranslatedList translationText translationsFilter untranslatedFilter newerKeys untranslated translations '
	classVariableNames: 'CheckMethods '
	poolDictionaries: ''
	category: 'Multilingual-Editor'!

!LanguageEditor methodsFor: 'accessing' stamp: 'tak 3/13/2006 20:23'!
translations
	"answet the translator's translations"
	| allTranslations filterString |
	translations ifNotNil: [^translations].
	allTranslations := self translator translations keys.
	""
	filterString := self translationsFilter.
	""
	filterString isEmpty
		ifFalse: [allTranslations := allTranslations
						select: [:each | ""
							('*' , filterString , '*' match: each)
								or: ['*' , filterString , '*'
										match: (self translator translationFor: each)]]].
""
	^ translations _ allTranslations asSortedCollection asArray! !

!LanguageEditor methodsFor: 'accessing' stamp: 'tak 3/13/2006 20:24'!
untranslated
	"answer the translator's untranslated phrases"
	

	| all filterString |
	untranslated ifNotNil: [^ untranslated].
	all := self translator untranslated.
	""
	filterString := self untranslatedFilter.
	""
	filterString isEmpty
		ifFalse: [all := all
						select: [:each | ""
							('*' , filterString , '*' match: each)
								or: ['*' , filterString , '*'
										match: (self translator translationFor: each)]]].
	""
	^ untranslated _ all asSortedCollection asArray! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 3/13/2006 20:17'!
browseMethodsWithUntranslated
	| untrans |
	self selectedUntranslated isZero
		ifTrue: [""
			self beep.
			self inform: 'select the untrans phrase to look for' translated.
			^ self].
	""
	untrans := self untranslated at: self selectedUntranslated.
	SystemNavigation default browseMethodsWithLiteral: untrans.
! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 3/13/2006 20:36'!
filterTranslations: aString 
| filter |
filter := aString ifNil:[''].
""
	translationsFilter _ filter.
	self refreshTranslations.
! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 3/13/2006 20:30'!
filterUntranslated: aString 
	| filter |
	filter := aString
				ifNil: [''].
	""
	untranslatedFilter := filter.
	self refreshUntranslated! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 3/13/2006 20:41'!
loadFromFile
	| fileName |
	fileName := self selectTranslationFileName.
	fileName isNil
		ifTrue: [""
			self beep.
			^ self].
	""
	Cursor wait
		showWhile: [
			self translator loadFromFileNamed: fileName.
			self refreshBoth]! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 3/13/2006 20:41'!
mergeFromFile
	| fileName |
	fileName := self selectTranslationFileName.
	fileName isNil
		ifTrue: [""
			self beep.
			^ self].
	""
	Cursor wait
		showWhile: [
			self translator loadFromFileNamed: fileName.
			self refreshBoth]! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 3/13/2006 20:41'!
phrase: phraseString translation: translationString 
	"set the models's translation for phraseString"
	self translator phrase: phraseString translation: translationString.
	self refreshBoth.

	newerKeys add: phraseString.
! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 3/13/2006 20:41'!
removeTranslation
	"remove the selected translation"
	| translation |
	self selectedTranslation isZero
		ifTrue: [""
			self beep.
			self inform: 'select the translation to remove' translated.
			^ self].
	""
	translation := self translations at: self selectedTranslation.
""
	(self
			confirm: ('Removing "{1}".
Are you sure you want to do this?' translated format: {translation}))
		ifFalse: [^ self].
""
	self translator removeTranslationFor: translation.
	self refreshBoth! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 3/13/2006 20:18'!
removeUntranslated
	"remove the selected untranslated phrase"
	| untrans |
	self selectedUntranslated isZero
		ifTrue: [""
			self beep.
			self inform: 'select the untranslated phrase to remove' translated.
			^ self].
	""
	untrans := self untranslated at: self selectedUntranslated.
	""
	(self
			confirm: ('Removing "{1}".
Are you sure you want to do this?' translated format: {untrans}))
		ifFalse: [^ self].
	""
	self translator removeUntranslated: untrans! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 3/13/2006 20:52'!
searchTranslation: aString 
	| results index |
	results := self translations
				select: [:each | ""
					('*' , aString , '*' match: each)
						or: ['*' , aString , '*' match: (self translator translationFor: each)]].
	""
	results isEmpty
		ifTrue: [""
			self inform: 'no matches for' translated , ' ''' , aString , ''''.
			^ self].
	""
	results size = 1
		ifTrue: [""
			self selectTranslationPhrase: results first.
			^ self].
	""
	index := (PopUpMenu
				labelArray: (results
						collect: [:each | ""
							(each copy replaceAll: Character cr with: $\)
								, ' -> '
								, ((self translator translationFor: each) copy replaceAll: Character cr with: $\)]))
				startUpWithCaption: 'select the translation...' translated.
	""
	index isZero
		ifTrue: [""
			self beep.
			^ self].
	""
	self
		selectTranslationPhrase: (results at: index)! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 3/13/2006 20:53'!
selectNewerKeys

	| index |
	self deselectAllTranslation.
	newerKeys do: [:k |
		index _ self translations indexOf: k ifAbsent: [0].
		index > 0 ifTrue: [
			self selectedTranslationsAt: index put: true
		].
	].
! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 3/22/2006 19:22'!
status
	"answer a status string"
	| translationsSize untranslatedSize |
	translationsSize := self translator translations size.
	untranslatedSize := self translator untranslated size.
	^ '| {1} phrases | {2} translated | {3} untranslated |' translated format: {translationsSize + untranslatedSize. translationsSize. untranslatedSize}! !

!LanguageEditor methodsFor: 'gui methods' stamp: 'tak 3/13/2006 20:42'!
translate
	"translate a phrase"
	| phrase |
	phrase := self phraseToTranslate.
	""
	(phrase isNil
			or: [phrase = ''])
		ifTrue: [""
			self beep.
			^ self].
	""
	self translatePhrase: phrase.
	self refreshBoth! !

!LanguageEditor methodsFor: 'stef' stamp: 'tak 3/13/2006 20:30'!
removeUntranslatedButUnusedStrings
	(self confirm: 'Are you sure that you want to remove unused strings?' translated)
		ifFalse: [^ self].
	untranslatedList getList
		do: [:each | 
			| timesUsed | 
			timesUsed := self numberOfTimesStringIsUsed: each.
			Transcript show: each.
			Transcript show: timesUsed printString;
				 cr.
			timesUsed isZero 
				ifTrue: [self translator removeUntranslated: each]].

	self refreshUntranslated.
! !

!LanguageEditor methodsFor: 'updating' stamp: 'tak 3/13/2006 20:40'!
refreshBoth
	self refreshUntranslated
! !

!LanguageEditor methodsFor: 'updating' stamp: 'tak 3/13/2006 20:40'!
refreshTranslations
	"refresh the translations panel"
	self selectedTranslation: 0.
	translations := nil.
	self changed: #translations.
! !

!LanguageEditor methodsFor: 'updating' stamp: 'tak 3/13/2006 20:39'!
refreshUntranslated
"refresh the untranslated panel"
	self refreshTranslations.
	self selectedUntranslated: 0.
	untranslated := nil.
	self changed: #untranslated.
! !

!LanguageEditor methodsFor: 'private' stamp: 'tak 3/13/2006 20:47'!
check
	"check the translations and answer a collection with the results"
	| results counter phrasesCount checkMethod |
	results := OrderedCollection new.
	untranslated := self untranslated.
	phrasesCount := self translations size + self untranslated size.
	counter := 0.
	checkMethod := self class checkMethods at: self translator localeID printString ifAbsent: [^results].
	
	self translations
		keysAndValuesDo: [:phrase :translation | 
			| result | 
			result := self perform: checkMethod with: phrase with: translation.
			(result notNil
					and: [result notEmpty])
				ifTrue: [results add: {phrase. translation. result}].
		
			counter := counter + 1.
			(counter isDivisibleBy: 50)
				ifTrue: [| percent | 
					percent := counter / phrasesCount * 100 roundTo: 0.01.
					Transcript
						show: ('- checked {1} phrases of {2} ({3}%)...' translated format: {counter. phrasesCount. percent});
						 cr]].

	self untranslated
		do: [:phrase | 
			| result | 
			result := self checkUntranslatedPhrase: phrase.
			(result notNil
					and: [result notEmpty])
				ifTrue: [results add: {phrase. nil. result}].
		
			counter := counter + 1.
			(counter isDivisibleBy: 50)
				ifTrue: [| percent | 
					percent := counter / phrasesCount * 100 roundTo: 0.01.
					Transcript
						show: ('- checked {1} phrases of {2} ({3}%)...' translated format: {counter. phrasesCount. percent});
						 cr]].

	^ results! !


!NaturalLanguageTranslator methodsFor: 'translation' stamp: 'tak 3/22/2006 12:37'!
translate: aString 
	^ self generics
		at: aString
		ifAbsent: [self localeID hasParent
				ifTrue: [self class translate: aString to: self localeID parent]
				ifFalse: [aString]]! !

SystemWindow subclass: #LanguageEditor
	instanceVariableNames: 'translator translations untranslated selectedTranslation selectedTranslations selectedUntranslated translationsList untranslatedList translationText translationsFilter untranslatedFilter newerKeys'
	classVariableNames: 'CheckMethods'
	poolDictionaries: ''
	category: 'Multilingual-Editor'!
